import Callable from './Callable';
import ContextualBindingBuilder from './ContextualBindingBuilder';
import IContainer from '../Contracts/Container/IContainer';
import ReflectionParameter from '../Reflection/ReflectionParameter';
import { Binding, Identifier, Instantiable } from '../Support/types';
declare class Container implements IContainer {
    protected static _instance?: Container;
    _contextual: any;
    protected _resolved: Map<any, boolean>;
    protected _bindings: Map<any, Binding>;
    protected _methodBindings: Map<string, Function>;
    protected _instances: Map<any, any>;
    protected _aliases: Map<any, any>;
    protected _abstractAliases: Map<any, any[]>;
    protected _extenders: Map<any, Function[]>;
    protected _tags: Map<string, any[]>;
    protected _buildStack: any[];
    protected _with: Array<any[] | object>;
    protected _reboundCallbacks: Map<any, Function[]>;
    protected _globalResolvingCallbacks: Function[];
    protected _globalAfterResolvingCallbacks: Function[];
    protected _resolvingCallbacks: Map<any, Function[]>;
    protected _afterResolvingCallbacks: Map<any, Function[]>;
    static getInstance(): Container;
    static setInstance(container?: Container): Container | undefined;
    private static _formatName;
    when<T>(concrete: Instantiable<T>[] | Instantiable<T>): ContextualBindingBuilder;
    bound<T>(abstract: Identifier<T>): boolean;
    has<T>(id: Identifier<T>): boolean;
    resolved<T>(abstract: Identifier<T>): boolean;
    isShared<T>(abstract: Identifier<T>): boolean;
    isAlias<T>(name: Identifier<T>): boolean;
    bind<U, V>(abstract: Identifier<U>, concrete?: Identifier<V> | Function, shared?: boolean): void;
    unbind<T>(abstract: Identifier<T>): void;
    hasMethodBinding(method: string): boolean;
    bindMethod<T>(method: [Instantiable<T>, string] | string, callback: Function): void;
    callMethodBinding(method: string, instance: any): any;
    addContextualBinding<T>(concrete: any, abstract: Identifier<T>, implementation: any): void;
    bindIf<U, V>(abstract: Identifier<U>, concrete?: Identifier<V> | Function, shared?: boolean): void;
    singleton<U, V>(abstract: Identifier<U>, concrete?: Identifier<V> | Function): void;
    extend<T>(abstract: Identifier<T>, closure: Function): void;
    instance<U, V>(abstract: Identifier<U>, instance: V): V;
    tag<T>(abstracts: Identifier<T>[] | Identifier<T>, tags: string[]): void;
    tagged(tag: string): any[];
    alias<U, V>(abstract: Identifier<U>, alias: Identifier<V>): void;
    rebinding<T>(abstract: Identifier<T>, callback: Function): unknown | undefined;
    refresh<T>(abstract: Identifier<T>, target: object, method: string): unknown;
    wrap<T>(callback: Callable<T>, parameters?: any[] | object): Function;
    call<T>(callback: Callable<T>, parameters?: any[] | object, defaultMethod?: string): any;
    factory<T>(abstract: Identifier<T>): Function;
    make<T>(abstract: Identifier<T>, parameters?: any[] | object): any;
    get<T>(id: Identifier<T>): any;
    set<U, V>(id: Identifier<U>, value: V): void;
    build<T>(concrete: Instantiable<T> | Function): any;
    resolving<T>(abstract: Identifier<T> | Function, callback?: Function): void;
    afterResolving<T>(abstract: Identifier<T> | Function, callback: Function): void;
    getBindings(): Map<any, Binding>;
    getAlias<T>(abstract: Identifier<T>): Identifier<any>;
    forgetExtenders<T>(abstract: Identifier<T>): void;
    forgetInstance<T>(abstract: Identifier<T>): void;
    forgetInstances(): void;
    flush(): void;
    protected _getClosure<U, V>(abstract: Identifier<U>, concrete: Identifier<V>): Function;
    protected _parseBindMethod<T>(method: [T, string] | string): string;
    protected _removeAbstractAlias<T>(searched: Identifier<T>): void;
    protected _rebound<T>(abstract: Identifier<T>): void;
    protected _getReboundCallbacks<T>(abstract: Identifier<T>): Function[];
    protected _resolve<T>(abstract: Identifier<T>, parameters?: any[] | object): any;
    protected _getConcrete<T>(abstract: Identifier<T>): Identifier<T> | any;
    protected _getContextualConcrete<T>(abstract: Identifier<T>): any;
    protected _findInContextualBindings<T>(abstract: Identifier<T>): any;
    protected _isBuildable<U, V>(concrete: Identifier<U> | U, abstract: Identifier<V>): boolean;
    protected _resolveDependencies(dependencies: ReflectionParameter[]): any[];
    protected _hasParameterOverride(dependency: ReflectionParameter): boolean;
    protected _getParameterOverride(dependency: ReflectionParameter): any;
    protected _getLastParameterOverride(): any[] | object;
    protected _resolvePrimitive(parameter: ReflectionParameter): any | undefined;
    protected _resolveClass(parameter: ReflectionParameter): any;
    protected _notInstantiable<T>(concrete: T): void;
    protected _unresolvablePrimitive(parameter: ReflectionParameter): void;
    protected _fireResolvingCallbacks<T>(abstract: Identifier<T>, object: object): void;
    protected _fireAfterResolvingCallbacks<T>(abstract: Identifier<T>, object: object): void;
    protected _getCallbacksForType<T>(abstract: Identifier<T>, object: object, callbacksPerType: Map<string, Function[]>): Function[];
    protected _fireCallbackArray(object: object, callbacks: Function[]): void;
    protected _getExtenders<T>(abstract: Identifier<T>): Function[];
    protected _dropStaleInstances<T>(abstract: Identifier<T>): void;
}
export default Container;
